﻿using PlcSiemens.Core.Extension;
using ServiceCenter;
using WCS.Core;
using WCS.WorkEngineering.Extensions;
using WCS.WorkEngineering.Protocol.BCR;
using WCS.WorkEngineering.Protocol.RGV;
using WCS.WorkEngineering.Protocol.SRM;
using WCS.WorkEngineering.Protocol.Station;
using WCS.WorkEngineering.Protocol.Truss;
using DeviceFlags = WCS.WorkEngineering.Extensions.DeviceFlags;

namespace WCS.WorkEngineering
{
    /// <summary>
    /// 业务工程配置信息
    /// </summary>
    public static class WorkStart
    {
        /// <summary>
        ///  初始化 设备信息
        /// </summary>

        public static void InitializeDeviceInfo()
        {
            #region 设置设备IP集合

            ServiceHub.DeviceIPList = new List<string>() { "10.30.43.90" };

            #endregion 设置设备IP集合

            #region 初始化输送机相关信息

            #region 基本信息

            #region 库尾输送线

            List<StationSegmentInfo> segmentInfo = new List<StationSegmentInfo>
            {
                new StationSegmentInfo(8271, 8279, "10.30.43.90"),
                new StationSegmentInfo(8161, 8270, "10.30.43.93")
            };

            foreach (var item in segmentInfo)
            {
                for (int i = item.Start; i <= item.End; i++)
                {
                    var conv = new Device(i.ToString());
                    conv.AddFlag(DeviceFlags.输送机);
                    conv.AddProtocol<IStation520>((i - item.Start) * 14, 520, item.IP);
                    conv.AddProtocol<IStation521>((i - item.Start) * 16, 521, item.IP);
                    conv.AddProtocol<IStation523>((i - item.Start) * 12, 523, item.IP);
                }
            }

            #endregion 库尾输送线

            #endregion 基本信息

            #region 托盘线扫码器

            List<BcrInfo> bcrInfo = new List<BcrInfo>
            {
                new BcrInfo(new int[] {8277}, "10.30.43.90"),
                new BcrInfo(new int[] {8270}, "10.30.43.93")
            };

            foreach (var item in bcrInfo)
            {
                for (int i = 0; i < item.DeviceNo.Length; i++)
                {
                    var device = Device.All.FirstOrDefault(v => v.Code == item.DeviceNo[i].ToString());
                    device.AddFlag(DeviceFlags.扫码);
                    int pos = i * 20;
                    device.AddProtocol<IBCR81>(pos, 81, item.IP);
                }
            }

            #endregion 托盘线扫码器

            #region 外检信息
            //要按先后顺序，后面添加外检要注意
            List<ShapeInfo> shapeInfo = new List<ShapeInfo>
            { new ShapeInfo(new int[] { 8277 }, "10.30.43.90"), 
              new ShapeInfo(new int[] { 8270 }, "10.30.43.93") };

            foreach (var item in shapeInfo)
            {
                for (int i = 0; i < item.DeviceNo.Length; i++)
                {
                    var conv = Device.All.FirstOrDefault(x => x.Code == item.DeviceNo[i].ToString());
                    conv.AddFlag(DeviceFlags.外检);
                    int pos = i * 14;
                    conv.AddProtocol<IStation91>(pos, 91, item.IP);
                }
            }

            #endregion 外检信息

            #endregion 初始化输送机相关信息

            #region 初始化RGV相关信息

            List<RGVSegmentInfo> RgvInfo = new List<RGVSegmentInfo>
            {
                //new RGVSegmentInfo( 1, "10.30.37.113"),
                //new RGVSegmentInfo( 2, "10.30.37.118"),
                //new RGVSegmentInfo( 3, "10.30.37.123"),
                //new RGVSegmentInfo( 4, "10.30.37.128"),
                //new RGVSegmentInfo( 5, "10.30.37.133"),
                //new RGVSegmentInfo( 6, "10.30.37.138")
            };

            foreach (var item in RgvInfo)
            {
                var conv = new Device($"RGV{item.Code}");
                conv.AddFlag(DeviceFlags.RGV);
                conv.AddProtocol<IRGV520>(0, 520, item.IP);
                conv.AddProtocol<IRGV521>(0, 521, item.IP);
                //conv.AddProtocol<IStation523>(, 523, item.IP);
            }

            #endregion 初始化RGV相关信息            

            #region 初始化堆垛机相关信息

            int ip = 21;

            for (int i = 0; i <= 1; i++)
            {
                var srm = new Device($"SRM{i + 1}");
                srm.AddFlag(DeviceFlags.堆垛机);
                ip = i == 0 ? ip : ip + 10;
                srm.AddProtocol<ISRM520>(0, 520, $"10.30.43.{ip}");
                srm.AddProtocol<ISRM521>(0, 521, $"10.30.43.{ip}");
                srm.AddProtocol<ISRM537>(0, 537, $"10.30.43.{ip}");

                //增加巷道
                var tunnel = new Device($"TY{i + 1}");
                tunnel.AddFlag(DeviceFlags.巷道);
            }

            #endregion 初始化堆垛机相关信息

            #region 配置路径信息

            List<RouteInfo> routeInfos = new List<RouteInfo>
            {
                    new RouteInfo("SRM1", new string[] { "TY1" }),
                    new RouteInfo("SRM2", new string[] { "TY2" }),
                    new RouteInfo("SRM3", new string[] { "TY3" }),
                    new RouteInfo("SRM4", new string[] { "TY4" }),
                    new RouteInfo("SRM5", new string[] { "TY5" }),
                    new RouteInfo("TY1", new string[] { "SRM1" }),
                    new RouteInfo("TY2", new string[] { "SRM2" }),
                    new RouteInfo("TY3", new string[] { "SRM3" }),
                    new RouteInfo("TY4", new string[] { "SRM4" }),
                    new RouteInfo("TY5", new string[] { "SRM5" }),

                    //入
                    new RouteInfo("TY1", new string[] { "8271" ,"8250"}),
                    new RouteInfo("TY2", new string[] { "8272" ,"8249"}),
                    new RouteInfo("TY3", new string[] { "8273" ,"8246"}),
                    new RouteInfo("TY4", new string[] { "8274" ,"8244"}),
                    new RouteInfo("TY5", new string[] { "8275" ,"8243"}),
                    //出
                    new RouteInfo("8271", new string[] { "TY1" }),
                    new RouteInfo("8272", new string[] { "TY2" }),
                    new RouteInfo("8273", new string[] { "TY3" }),
                    new RouteInfo("8274", new string[] { "TY4" }),
                    new RouteInfo("8275", new string[] { "TY5" }),
                    new RouteInfo("8251", new string[] { "TY1" }),
                    new RouteInfo("8248", new string[] { "TY2" }),
                    new RouteInfo("8247", new string[] { "TY3" }),
                    new RouteInfo("8245", new string[] { "TY4" }),
                    new RouteInfo("8242", new string[] { "TY5" }),

                    //new RouteInfo("8271", new string[] { "8278" }),
                    //new RouteInfo("8272", new string[] { "8278" }),
                    //new RouteInfo("8273", new string[] { "8278" }),
                    //new RouteInfo("8274", new string[] { "8278" }),
                    //new RouteInfo("8275", new string[] { "8278" }),


            };

            foreach (var item in routeInfos)
            {
                var device = Device.All.FirstOrDefault(x => x.Code == item.DeviceCode);
                device.AddTarget(item.NextList);

                foreach (var next in item.NextList)
                {
                    var dev = Device.All.FirstOrDefault(x => x.Code == next);
                    dev.Parent = device;
                }
            }

            #endregion 配置路径信息

            #region 标签配置

            Dictionary<DeviceFlags, List<string>> devices = new Dictionary<DeviceFlags, List<string>>
            {
                { DeviceFlags.入库, new List<string>() { "8271", "8272", "8273", "8274", "8275", "8250", "8249", "8246", "8244", "8243"} },
                { DeviceFlags.出库, new List<string>() { "8271", "8272", "8273", "8274", "8275" } },
                { DeviceFlags.扫码, new List<string>() {"8277"} },
                { DeviceFlags.出库口, new List<string>() { "8271","8272", "8273", "8274", "8275" } },
                { DeviceFlags.出货口, new List<string>() { "8263", "8257", "8269" } }
        };

            devices.ForEach(item =>
            {
                item.Value.ForEach(code =>
                {
                    var device = Device.All.FirstOrDefault(v => v.Code == code);
                    device.AddFlag(item.Key);
                });
            });

            #endregion 标签配置
        }
    }
}
