﻿using ServiceCenter.Extensions;
using ServiceCenter.Logs;
using ServiceCenter.SqlSugars;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using WCS.Core;
using WCS.Entity;
using WCS.WorkEngineering.Extensions;
using WCS.WorkEngineering.Protocol.BCR;
using WCS.WorkEngineering.Protocol.SRM;
using WCS.WorkEngineering.Protocol.Station;
using WCS.WorkEngineering.Worlds;
using DeviceFlags = WCS.WorkEngineering.Extensions.DeviceFlags;

namespace WCS.WorkEngineering.Systems
{
    /// <summary>
    ///  一楼入库工位处理系统
    /// </summary>
    [BelongTo(typeof(MainWorld))]
    [Description("一楼入库工位处理系统")]
    public class 一楼入库工位处理系统 : DeviceSystem<Station>
    {
        protected override bool ParallelDo => true;

        protected override bool SaveLogsToFile => true;

        //private List<BCR> BCRS = new List<BCR>();

        private List<Station> devs = new List<Station>();
        private List<SRM> srms = new List<SRM>();

        public 一楼入库工位处理系统()
        {
            //BCRS = Device.All.Where(v => v.HasProtocol<IBCR81>()).Select(v => new BCR(v, World)).ToList();
            devs = Device.All.Where(v => v.HasFlag(DeviceFlags.入库, DeviceFlags.巷道口)).Select(v => new Station(v, this.World)).ToList();
            srms = Device.All.Where(v => v.HasProtocol(typeof(ISRM520))).Select(v => new SRM(v, this.World)).ToList();
        }

        public override void Do(Station obj)
        {
            obj.入库站点是否被禁止();
            obj.入库站点是否满足执行条件();            
            WCS_TaskInfo taskInfo = null;
            try
            {
                SqlSugarHelper.Do(_db =>
                {
                    var db = _db.Default;                  
                    if (!obj.Data3.Status.HasFlag(StationStatus.PH_Status) || !obj.Data3.Status.HasFlag(StationStatus.OT_Status))
                        return;
                    if (!obj.Data3.Status.HasFlag(StationStatus.Auto))
                        return;
                    taskInfo = db.Queryable<WCS_TaskInfo>().Where(v => v.ID == obj.Data2.TaskNumber && v.Status < Entity.TaskStatus.Finish).First();
                    if(taskInfo == null) throw new KnownException($"未找到对应wcs任务{obj.Data2.TaskNumber}", LogLevelEnum.High);
                    if (taskInfo.SrmStation == obj.Entity.Code && taskInfo.Status == Entity.TaskStatus.AGVExecution)
                    {
                        taskInfo.Status = Entity.TaskStatus.ConveyorExecution;
                        taskInfo.EditTime = DateTime.Now;
                        db.Updateable(taskInfo).ExecuteCommand();
                        //货架信号
                        var dev = devs.Find(v => v.Entity.Code == taskInfo.SrmStation);
                        dev.Data.GoodsStart = dev.Entity.Code.ToShort();                       
                    }
                });
            }
            catch (Exception ex)
            {
                throw new KnownException(ex.Message, LogLevelEnum.High);
            }            
        }
        public override bool Select(Device dev)
        {
            return dev.HasFlag(DeviceFlags.一楼入库口);
        }
    }
}
