﻿using PlcSiemens.Core.Extension;
using ServiceCenter;
using WCS.Core;
using WCS.WorkEngineering.Extensions;
using WCS.WorkEngineering.Protocol.BCR;
using WCS.WorkEngineering.Protocol.RGV;
using WCS.WorkEngineering.Protocol.SRM;
using WCS.WorkEngineering.Protocol.Station;
using WCS.WorkEngineering.Protocol.Truss;
using DeviceFlags = WCS.WorkEngineering.Extensions.DeviceFlags;

namespace WCS.WorkEngineering
{
    /// <summary>
    /// 业务工程配置信息
    /// </summary>
    public static class WorkStart
    {
        /// <summary>
        ///  初始化 设备信息
        /// </summary>

        public static void InitializeDeviceInfo()
        {
            #region 设置设备IP集合

            ServiceHub.DeviceIPList = new List<string>() {};

            #endregion 设置设备IP集合

            #region 初始化输送机相关信息

            #region 基本信息

            List<StationSegmentInfo> segmentInfo = new List<StationSegmentInfo>
            {
                new StationSegmentInfo(1001, 1026,"10.30.35.21")
            };

            foreach (var item in segmentInfo)
            {
                for (int i = item.Start; i <= item.End; i++)
                {
                    var conv = new Device(i.ToString());
                    conv.AddFlag(DeviceFlags.输送机);
                    conv.AddProtocol<IStation520>((i - item.Start) * 14, 520, item.IP);
                    conv.AddProtocol<IStation521>((i - item.Start) * 16, 521, item.IP);
                    conv.AddProtocol<IStation523>((i - item.Start) * 12, 523, item.IP);
                }
            }
            #endregion 基本信息
            #region 扫码器
            List<BcrInfo> bcrInfo = new List<BcrInfo>
            {
                new BcrInfo(new int[] { 1017,1018 }, "10.30.35.21")
            };

            foreach (var item in bcrInfo)
            {
                for (int i = 0; i < item.DeviceNo.Length; i++)
                {
                    var conv = new Device("BCR" + item.DeviceNo[i]);
                    conv.AddFlag(DeviceFlags.扫码);
                    int pos = i * 20;
                    conv.AddProtocol<IBCR81>(pos, 81, item.IP);
                }
            }

            #endregion 扫码器

            #region 外检信息
           

            #endregion 外检信息

            #endregion 初始化输送机相关信息           

            #region 初始化堆垛机相关信息
            int ip = 31;
            for (int i = 0; i <= 1; i++)
            {
                var srm = new Device($"SRM{i + 1}");
                srm.AddFlag(DeviceFlags.堆垛机);
                ip = i == 0 ? ip : ip + 10;
                //三台堆垛机IP主机位分别是 21 31 41
                srm.AddProtocol<ISRM520>(0, 520, $"10.30.35.{ip}");
                srm.AddProtocol<ISRM521>(0, 521, $"10.30.35.{ip}");
                srm.AddProtocol<ISRM537>(0, 537, $"10.30.35.{ip}");
            }

            for (int i = 0; i <= 3; i = i + 1)
            {
                //增加巷道
                var tunnel = new Device($"TY{i + 1}");
                tunnel.AddFlag(DeviceFlags.巷道);
            }

            #endregion 初始化堆垛机相关信息

            #region 配置路径信息

            List<RouteInfo> routeInfos = new List<RouteInfo>
            {
                new RouteInfo("SRM1", new string[] { "TY1"}),
                new RouteInfo("SRM1", new string[] { "TY2"}),
                new RouteInfo("SRM2", new string[] { "TY3"}),
                new RouteInfo("SRM2", new string[] { "TY4"}),
                //new RouteInfo("SRM3", new string[] { "TY5"}),
                //new RouteInfo("SRM3", new string[] { "TY6" }),
                //new RouteInfo("SRM4", new string[] { "TY7"}),
                //new RouteInfo("SRM4", new string[] { "TY8" }),
                new RouteInfo("TY1", new string[] { "SRM1" }),
                new RouteInfo("TY2", new string[] { "SRM1" }),
                new RouteInfo("TY3", new string[] { "SRM2" }),
                new RouteInfo("TY4", new string[] { "SRM2" }),
                //new RouteInfo("TY5", new string[] { "SRM3" }),
                //new RouteInfo("TY6", new string[] { "SRM3" }),
                //new RouteInfo("TY7", new string[] { "SRM4" }),
                //new RouteInfo("TY8", new string[] { "SRM4" }),
                //入
                new RouteInfo("TY1", new string[] { "1001"}),
                new RouteInfo("TY2", new string[] { "1003" }),
                new RouteInfo("TY3", new string[] { "1005"}),
                new RouteInfo("TY4", new string[] { "1007" }),
                //new RouteInfo("TY5", new string[] { "1009" }),
                //new RouteInfo("TY6", new string[] { "1011" }),
                //new RouteInfo("TY7", new string[] { "1013" }),
                //new RouteInfo("TY8", new string[] { "1015" }),
                //出
                new RouteInfo("1002", new string[] { "TY1" }),
                new RouteInfo("1004", new string[] { "TY2" }),
                new RouteInfo("1006", new string[] { "TY3" }),
                new RouteInfo("1008", new string[] { "TY4" }),
                //new RouteInfo("1010", new string[] { "TY5" }),
                //new RouteInfo("1012", new string[] { "TY6" }),
                //new RouteInfo("1014", new string[] { "TY7" }),
                //new RouteInfo("1016", new string[] { "TY8" })
            };

            foreach (var item in routeInfos)
            {
                var device = Device.All.FirstOrDefault(x => x.Code == item.DeviceCode);
                device.AddTarget(item.NextList);

                foreach (var next in item.NextList)
                {
                    var dev = Device.All.FirstOrDefault(x => x.Code == next);
                    dev.Parent = device;
                }
            }

            #endregion 配置路径信息

            #region 标签配置

            Dictionary<DeviceFlags, List<string>> devices = new Dictionary<DeviceFlags, List<string>>
            {
                { DeviceFlags.巷道口, new List<string>() { "1001", "1002", "1003", "1004", "1005", "1006", "1007", "1008", "1009", "1010", "1011", "1012","1013","1014","1015","1016"} },
                { DeviceFlags.入库, new List<string>() { "1001", "1003", "1005", "1007", "1009", "1011","1013","1015" } },
                { DeviceFlags.出库, new List<string>() { "1002", "1004", "1006", "1008", "1010", "1012","1014","1016" } },
                //{ DeviceFlags.扫码, new List<string>() { "1017", "1018" } },
                { DeviceFlags.一楼出库口, new List<string>() { "1002", "1004" , "1006", "1008" } },//  "1006", "1008", "1010", "1012", "1014", "1016" } },                
                { DeviceFlags.一楼入库口, new List<string>() { "1001", "1003", "1005","1007"} },//,"1009","1011","1013","1015" } },
                { DeviceFlags.一楼出入库口, new List<string>() { "1001", "1003", "1005","1007"} },//,"1009","1011","1013","1015" } },
                { DeviceFlags.检测门, new List<string>() { "1017", "1018" } },               
                { DeviceFlags.货架调度, new List<string>() { "1001", "1002" } },
                { DeviceFlags.叫料站台, new List<string>() { "1019","1020","1021","1022","1023","1024" } },

        };
            devices.ForEach(item =>
            {
                item.Value.ForEach(code =>
                {
                    var device = Device.All.FirstOrDefault(v => v.Code == code);
                    device.AddFlag(item.Key);
                });
            });

            #endregion 标签配置
        }
    }
}
